/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.plugin.layoutComponents;

import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

import eu.DECIDEh2020.architect.appManager.models.Microservice;

import java.util.ArrayList;
import java.util.HashSet;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.apache.jena.rdf.model.Resource;
import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.core.databinding.beans.PojoProperties;

public class MicroserviceComposite extends Composite {
	private DataBindingContext m_bindingContext;

	private static class DependecyLabelProvider extends LabelProvider {
		public Image getImage(Object element) {
			return null;
		}

		public String getText(Object element) {
			return (String) element;
		}
	}

	private static class DependecyContentProvider implements IStructuredContentProvider {
		public Object[] getElements(Object inputElement) {
			return ((Microservice) inputElement).getDependencies().toArray();
		}

		public void dispose() {
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}

	Color background = SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND);
	private Button btnStateless;
	private Microservice microservice;
	private java.util.List<ContentChangeListener> contentChangeListeners = new ArrayList<ContentChangeListener>();
	private Text txtAddDependency;
	private Text msRepo;

	
	
	
	/**
	 * Create the composite.
	 * 
	 * @param parent
	 * @param style
	 */
	public MicroserviceComposite(Composite parent, Microservice microservice, Color background) {
		super(parent, SWT.None);
		this.background = background;
		setBackground(background);
		this.microservice = microservice;

		this.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1, 1));
		this.setLayout(new FillLayout(SWT.HORIZONTAL));

		Group grpMicroservice = new Group(this, SWT.NONE);
		grpMicroservice.setBackground(background);
		grpMicroservice.setText(microservice.getName());
		grpMicroservice.setLayout(new FillLayout(SWT.HORIZONTAL));

		Composite composite = new Composite(grpMicroservice, SWT.NONE);
		composite.setBackground(background);
		GridLayout gl_composite = new GridLayout(2, false);
		gl_composite.marginLeft = 10;
		gl_composite.marginRight = 10;
		composite.setLayout(gl_composite);

		Label lblNewLabel = new Label(composite, SWT.NONE);
		lblNewLabel.setBackground(background);
		lblNewLabel.setText("Microservice Name:");

		Text textMsvName = new Text(composite, SWT.BORDER);
		textMsvName.addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				microservice.setName(textMsvName.getText());
				grpMicroservice.setText(microservice.getName());
				setContentChanged();
			}
		});
		textMsvName.setText(grpMicroservice.getText());
		textMsvName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		Label lblProgrammingLanguage = new Label(composite, SWT.NONE);
		lblProgrammingLanguage.setBackground(background);
		lblProgrammingLanguage.setText("Programming Language:");

		Combo comboProgrammingLanguage = new Combo(composite, SWT.NONE);
		comboProgrammingLanguage.addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {
				microservice.setLanguage(comboProgrammingLanguage.getText());
			}
		});

		comboProgrammingLanguage.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		comboProgrammingLanguage.setItems(new String[] { "Java", "Go", "Ruby" });

		// set combobox to language. if language not in Combobox, add it
		String msvLang = microservice.getLanguage();
		int index = -1;
		if (msvLang != null && !msvLang.isEmpty()) {
			index = comboProgrammingLanguage.indexOf(msvLang);
		}
		if (index >= 0) {
			comboProgrammingLanguage.select(index);
		} else if (index == -1 && !msvLang.isEmpty()) {
			comboProgrammingLanguage.add(msvLang, 0);
			comboProgrammingLanguage.select(0);
		}

		comboProgrammingLanguage.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
			
				microservice.setLanguage(comboProgrammingLanguage.getText());
			}
		});

		Label lblType = new Label(composite, SWT.NONE);
		lblType.setBackground(background);
		lblType.setText("Type:");

		Combo comboType = new Combo(composite, SWT.READ_ONLY);
		comboType.setItems(new String[] { "Backend", "Database" });
		String msvType = microservice.getType();
		if (msvType != null && !msvType.isEmpty()) {
			comboType.select(comboType.indexOf(msvType));
		}

		comboType.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {

				microservice.setType(comboType.getText());
			}
		});

		comboType.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		Label lblNfr = new Label(composite, SWT.NONE);
		lblNfr.setBackground(background);
		lblNfr.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
		lblNfr.setText("NFRs:");

		NfrListViewer nfrList = new NfrListViewer(composite, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI);

		// select all nfrs in this microservice
		if (microservice.getNfrs().size() != 0) {
			
			nfrList.addSelection(new ArrayList<Resource>(microservice.getNfrs()));
		}

		nfrList.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				HashSet<Resource> mscNfrs = new HashSet<Resource>();
				for (Resource r : nfrList.getSelection()) {
					try {
						mscNfrs.add(r);
					} catch (IllegalArgumentException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				}
				microservice.setNfrs(mscNfrs);
			}
		});

		GridData gd_nfrList = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_nfrList.widthHint = 448;
		nfrList.setLayoutData(gd_nfrList);

		Label lblDependencies = new Label(composite, SWT.NONE);
		lblDependencies.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
		lblDependencies.setText("Dependencies:");
		lblDependencies.setBackground(background);

		Composite dependenciesComposite = new Composite(composite, SWT.NONE);
		GridLayout gl_dependenciesComposite = new GridLayout(2, false);
		gl_dependenciesComposite.marginLeft = -5;
		dependenciesComposite.setLayout(gl_dependenciesComposite);
		dependenciesComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		dependenciesComposite.setBackground(background);

		ListViewer listViewer = new ListViewer(dependenciesComposite, SWT.BORDER | SWT.V_SCROLL);
		List dependencyList = listViewer.getList();

		GridData gd_dependencyList = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_dependencyList.minimumHeight = 85;
		gd_dependencyList.heightHint = 85;
		dependencyList.setLayoutData(gd_dependencyList);
		listViewer.setLabelProvider(new DependecyLabelProvider());
		listViewer.setContentProvider(new DependecyContentProvider());
		listViewer.setInput(microservice);

		Composite DepBttnComposite = new Composite(dependenciesComposite, SWT.NONE);
		DepBttnComposite.setLayoutData(new GridData(SWT.LEFT, SWT.BOTTOM, false, false, 1, 1));
		FillLayout fl_DepBttnComposite = new FillLayout(SWT.VERTICAL);
		fl_DepBttnComposite.spacing = 5;
		DepBttnComposite.setLayout(fl_DepBttnComposite);
		DepBttnComposite.setBackground(background);

		Button btnRemove = new Button(DepBttnComposite, SWT.NONE);
		btnRemove.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (dependencyList.getSelectionCount() == 1) {
					int i= dependencyList.getSelectionIndex();
					ArrayList<String> deps = microservice.getDependencies();
					deps.remove(i);
					listViewer.refresh();
				}
			}
		});
		btnRemove.setText("Remove");

		txtAddDependency = new Text(dependenciesComposite, SWT.BORDER);
		txtAddDependency.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		Button btnAdd = new Button(dependenciesComposite, SWT.NONE);
		btnAdd.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				microservice.addDependency(txtAddDependency.getText());
				listViewer.refresh();
				txtAddDependency.setText("");
			}
		});
		btnAdd.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		btnAdd.setText("Add");

		dependencyList.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (dependencyList.getSelectionCount() == 1) {
					btnRemove.setEnabled(true);
				}else {
					btnRemove.setEnabled(false);
				}
			}
		});

		new Label(composite, SWT.NONE);

		btnStateless = new Button(composite, SWT.CHECK);
		btnStateless.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		btnStateless.setSelection(microservice.isStateless());
		btnStateless.setBackground(background);
		btnStateless.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				microservice.setStateless(btnStateless.getSelection());
			}
		});
		btnStateless.setText("Stateless");

		Label lblMsRepo = new Label(composite, SWT.NONE);
		lblMsRepo.setBackground(background);
		lblMsRepo.setText("Microservice Repo : ");

		msRepo = new Text(composite, SWT.BORDER);
		msRepo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		new Label(composite, SWT.NONE);
/*
		Composite composite_msRepo_buttons = new Composite(composite, SWT.NONE);
		composite_msRepo_buttons.setBackground(background);
		FillLayout fl_composite_msRepo_buttons = new FillLayout(SWT.HORIZONTAL);
		fl_composite_msRepo_buttons.spacing = 5;
		composite_msRepo_buttons.setLayout(fl_composite_msRepo_buttons);

		Button btnAddNewRepo = new Button(composite_msRepo_buttons, SWT.NONE);
		btnAddNewRepo.setEnabled(false);
		btnAddNewRepo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				NewMscvWizard mscw = new NewMscvWizard();
				WizardDialog dialog = new WizardDialog(getShell(), mscw);
				if (dialog.open() == Window.OK) {
					msRepo.setText(mscw.getProject());
				} else {
					msRepo.setText("");

				}

			}
		});
		btnAddNewRepo.setText("Add New Microservice");*/
		m_bindingContext = initDataBindings();
		
		
	}

	/**
	 * Create the composite.
	 * 
	 * @param parent
	 * @param microservice
	 * @wbp.parser.constructor
	 */
	public MicroserviceComposite(Composite parent, Microservice microservice) {
		this(parent, microservice, SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));

	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

	public void addContentChangeListener(ContentChangeListener contentChangedListener) {
		contentChangeListeners.add(contentChangedListener);
	}

	public void setContentChanged() {
		for (ContentChangeListener ccl : contentChangeListeners) {
			ccl.contentChanged();
		}
	}

	protected DataBindingContext initDataBindings() {
		DataBindingContext bindingContext = new DataBindingContext();
		//
		IObservableValue observeTextMsRepoObserveWidget = WidgetProperties.text(SWT.Modify).observe(msRepo);
		IObservableValue repoMicroserviceObserveValue = PojoProperties.value("repo").observe(microservice);
		bindingContext.bindValue(observeTextMsRepoObserveWidget, repoMicroserviceObserveValue, null, null);
		//
		return bindingContext;
	}
}
