/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.plugin.layoutComponents;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.layout.GridData;
import org.eclipse.wb.swt.SWTResourceManager;

import eu.DECIDEh2020.architect.cloudpatterns.Patterns;

import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.widgets.List;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.custom.StyledText;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.vocabulary.DCTerms;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.swt.widgets.Table;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.CheckStateChangedEvent;


public class PatternPageComposite extends Composite {

	private Patterns patterns;
	private ListViewer listViewer_selected;
	private CheckboxTableViewer checkboxTableViewer_bad,checkboxTableViewer_good;
	private PatternDetailComposite patternDetailComposite;
	private Table table_bad, table_good;
	private java.util.List<Resource> selectedPatterns = new ArrayList<Resource>();
	
	private static class PatternLabelProvider extends LabelProvider {
		public Image getImage(Object element) {
			return null;
		}
		public String getText(Object element) {
			return  ((Resource) element).getProperty(DCTerms.title).getString();
		}
	}
	
	
	private static class PatternContentProvider implements IStructuredContentProvider {
		@SuppressWarnings("unchecked")
		public Object[] getElements(Object inputElement) {
			return ((java.util.List<Patterns>) inputElement).toArray();
			
		}
		public void dispose() {
		}
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}
	
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public PatternPageComposite(Composite parent, int style, Patterns patterns) {
		super(parent, style);
		this.patterns= patterns;
		createGUI();
		

	}
	private void createGUI() {
		setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		setLayout(new GridLayout(1, false));
		
		CLabel lblDesignPatterns = new CLabel(this, SWT.NONE);
		lblDesignPatterns.setFont(SWTResourceManager.getFont("Segoe UI", 12, SWT.BOLD));
		lblDesignPatterns.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		lblDesignPatterns.setText("Design Patterns");
		
		Label label = new Label(this, SWT.SEPARATOR | SWT.HORIZONTAL);
		label.setFont(SWTResourceManager.getFont("Segoe UI", 12, SWT.BOLD));
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		Composite composite_1 = new Composite(this, SWT.NONE);
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		composite_1.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridLayout gl_composite_1 = new GridLayout(2, true);
		gl_composite_1.horizontalSpacing = 25;
		composite_1.setLayout(gl_composite_1);
		
		Label lblSuggestedPatterns = new Label(composite_1, SWT.NONE);
		lblSuggestedPatterns.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
		lblSuggestedPatterns.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		lblSuggestedPatterns.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblSuggestedPatterns.setText("Suggested Patterns");
		
		Label lblPatternDetail = new Label(composite_1, SWT.NONE);
		lblPatternDetail.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
		lblPatternDetail.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblPatternDetail.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblPatternDetail.setText("Pattern detail");
		
		Composite composite_2 = new Composite(composite_1, SWT.NONE);
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		composite_2.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		composite_2.setLayout(new GridLayout(1, false));
		
		
		
		checkboxTableViewer_good = CheckboxTableViewer.newCheckList(composite_2, SWT.BORDER | SWT.FULL_SELECTION);
		table_good = checkboxTableViewer_good.getTable();
		
		table_good.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		checkboxTableViewer_good.setLabelProvider(new PatternLabelProvider());
		checkboxTableViewer_good.setContentProvider(new PatternContentProvider());
		checkboxTableViewer_good.setInput(patterns.listPatterns());
		
		
		Label lblbadPatterns = new Label(composite_2, SWT.NONE);
		lblbadPatterns.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblbadPatterns.setText("Bad Patterns");
		lblbadPatterns.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblbadPatterns.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
		
		
		
		checkboxTableViewer_bad = CheckboxTableViewer.newCheckList(composite_2, SWT.BORDER | SWT.FULL_SELECTION);
		checkboxTableViewer_bad.setAllGrayed(true);
		
		table_bad = checkboxTableViewer_bad.getTable();
		
		table_bad.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		checkboxTableViewer_bad.setLabelProvider(new PatternLabelProvider());
		checkboxTableViewer_bad.setContentProvider(new PatternContentProvider());
		checkboxTableViewer_bad.setInput(patterns.listPatterns());
		
		
		
		Label lblWhyNeccessary = new Label(composite_2, SWT.NONE);
		lblWhyNeccessary.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
		lblWhyNeccessary.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		lblWhyNeccessary.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblWhyNeccessary.setText("Selected Patterns");
		
		listViewer_selected = new ListViewer(composite_2, SWT.BORDER | SWT.V_SCROLL);
		List list_selected = listViewer_selected.getList();
		
		list_selected.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		list_selected.setBounds(0, 0, 88, 68);
		listViewer_selected.setLabelProvider(new PatternLabelProvider());
		listViewer_selected.setContentProvider(new PatternContentProvider());
		listViewer_selected.setInput(selectedPatterns);
		
		patternDetailComposite = new PatternDetailComposite(composite_1, SWT.NONE);
		patternDetailComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		
		checkboxTableViewer_good.addCheckStateListener(new ICheckStateListener() {
			public void checkStateChanged(CheckStateChangedEvent event) {
				if(event.getChecked()) {
					selectedPatterns.add((Resource) event.getElement());
				}else {
					
					selectedPatterns.remove(event.getElement());
				}
				listViewer_selected.refresh();
			}
		});
		
		list_selected.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				int i= list_selected.getSelectionIndex();
				if(i>=0) {
					Resource pattern = (Resource) listViewer_selected.getElementAt(i);
					patternDetailComposite.updatePattern(pattern);
					
				}
			}
		});
		
		table_bad.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				int i= table_bad.getSelectionIndex();
				if(i>=0) {
					Resource pattern = (Resource) checkboxTableViewer_bad.getElementAt(i);
					patternDetailComposite.updatePattern(pattern);
					
				}
			}
		});
		table_good.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				int i = table_good.getSelectionIndex();
				if(i>=0) {
					Resource pattern = (Resource) checkboxTableViewer_good.getElementAt(i);
					patternDetailComposite.updatePattern(pattern);
					
				}
			}
		});
		if(table_good.getItemCount()>0) {
			table_good.select(0);
			patternDetailComposite.updatePattern((Resource) checkboxTableViewer_good.getElementAt(0));
		}
	}
	
	
	
	public void updatePatterns(java.util.List<Resource> nfrs) {
		
		java.util.List<Resource> inferredPatterns = patterns.inferPatterns(nfrs);
		
		
		if(!inferredPatterns.equals((java.util.List<Resource>)checkboxTableViewer_good.getInput())) {
			checkboxTableViewer_good.setInput(inferredPatterns);
			selectedPatterns.clear();
			
			
			Object[] sr =   checkboxTableViewer_good.getCheckedElements();
			
			selectedPatterns.addAll((Collection<? extends Resource>) Arrays.asList(sr) );
			listViewer_selected.refresh();
			
		}
		
		
		
		checkboxTableViewer_bad.setInput(patterns.inferBadPatterns(nfrs));
		
		if(table_good.getItemCount()>0) {
			table_good.select(0);
			patternDetailComposite.updatePattern((Resource) checkboxTableViewer_good.getElementAt(0));
		}else {
		patternDetailComposite.updatePattern(null);
		}
		
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
}
