/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.appManager.models;


import java.util.HashSet;
import java.util.LinkedList;
import java.util.UUID;

import org.apache.jena.rdf.model.Resource;

public class AppDescriptor {
	private UUID id;
	private String name, description, version;
	private Boolean highTechnologicalRisk;
	
	private LinkedList<Microservice> microservices;
	private HashSet<Resource> projectNfrs;
	private Object app_mcsla,containers[], virtual_machines[];
	final public static String plugin_version ="0.2.2-alpha";
	
	public AppDescriptor() {
		id = UUID.randomUUID();
		microservices = new LinkedList<Microservice>();
		setDeploymentOrder();
		this.projectNfrs = new HashSet<Resource>();
		this.version = plugin_version;
	}
	
	
	
	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}



	/**
	 * @param description the description to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}



	/**
	 * @return the version
	 */
	public String getVersion() {
		return version;
	}



	/**
	 * @param version the version to set
	 */
	public void setVersion(String version) {
		this.version = version;
	}



	/**
	 * @return the highTechnologicalRisk
	 */
	public Boolean getHighTechnologicalRisk() {
		return highTechnologicalRisk;
	}



	/**
	 * @param highTechnologicalRisk the highTechnologicalRisk to set
	 */
	public void setHighTechnologicalRisk(Boolean highTechnologicalRisk) {
		this.highTechnologicalRisk = highTechnologicalRisk;
	}



	/**
	 * @return the app_mcsla
	 */
	public Object getApp_mcsla() {
		return app_mcsla;
	}



	/**
	 * @param app_mcsla the app_mcsla to set
	 */
	public void setApp_mcsla(Object app_mcsla) {
		this.app_mcsla = app_mcsla;
	}



	



	/**
	 * @return the containers
	 */
	public Object[] getContainers() {
		return containers;
	}



	/**
	 * @param containers the containers to set
	 */
	public void setContainers(Object[] containers) {
		this.containers = containers;
	}



	/**
	 * @return the virtual_machines
	 */
	public Object[] getVirtual_machines() {
		return virtual_machines;
	}



	/**
	 * @param virtual_machines the virtual_machines to set
	 */
	public void setVirtual_machines(Object[] virtual_machines) {
		this.virtual_machines = virtual_machines;
	}



	/**
	 * @param id the id to set
	 */
	public void setId(UUID id) {
		this.id = id;
	}



	public LinkedList<Microservice> changeIndex(int index, int plus) {
		
		int size = microservices.size();
		if (index+plus>=0&& index+plus<size) {
			Microservice mscv = microservices.get(index);
			microservices.remove(index);
			microservices.add(index+plus, mscv);
		}
		setDeploymentOrder();
		return microservices;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}



	

	public void setDeploymentOrder(){
		
		for (Microservice m: microservices) {
			m.setDeployment_order(microservices.indexOf(m));
			
		}
	}


	public LinkedList<Microservice> getMicroservices() {
		return microservices;
	}

	public void setMicroservices(LinkedList<Microservice> microservices) {
		this.microservices = microservices;
		setDeploymentOrder();
	}
	
	public LinkedList<Microservice> addMicroservice(Microservice microservice) {
		this.microservices.add(microservice);
		setDeploymentOrder();
		return this.microservices;
	}
	
	public LinkedList<Microservice> addMicroservices(int number){
		for (int j = 0; j < number; j++) {
			this.microservices.add(new Microservice());
		}
		setDeploymentOrder();
		return this.microservices;
	}
	
	public LinkedList<Microservice> removeMicroservice(int index) {
		this.microservices.remove(index);
		setDeploymentOrder();
		return this.microservices;
	}
	
	
	public LinkedList<Microservice> removeMicroservice(Microservice microservice) {
		this.microservices.remove(microservice);
		setDeploymentOrder();
		return this.microservices;
	}

	public String getIDString() {
		return id.toString();
	}
	
	/**
	 * @return the id
	 */
	public UUID getUID() {
		return id;
	}


	public HashSet<Resource> getProjectNfrs() {
		return projectNfrs;
	}



	public void setProjectNfrs(HashSet<Resource> projectNfrs) {
		this.projectNfrs = projectNfrs;
		
	}
	public void addProjectNfr(Resource nfr) {
		projectNfrs.add(nfr);
		
	}
	public void removeProjectNfr(Resource nfr) {
		projectNfrs.remove(nfr);
		
	}
}
