/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* GNU AGPL v3 which accompanies
* this distribution, and is available at
* http://www.gnu.org/licenses/agpl.txt
*
* Contributors:
*
* Simon Dutkowski Fraunhofer FOKUS
*
* Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
*/

package eu.DECIDEh2020.architect.cloudpatterns.compendium.converters;

import eu.DECIDEh2020.architect.cloudpatterns.DP;
import io.vertx.core.json.JsonArray;
import io.vertx.core.json.JsonObject;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.sparql.vocabulary.FOAF;
import org.apache.jena.vocabulary.DCTerms;

public class PatternConverter {

    public static Resource toResource(JsonObject nfr) {
        return null;
    }

    public static JsonObject toJson(Resource pattern) {
        JsonObject obj = new JsonObject();
        obj.put("uriref", pattern.getURI());
        obj.put("title", pattern.getRequiredProperty(DCTerms.title).getString());
        if (pattern.hasProperty(DCTerms.type)) {
            JsonArray array = new JsonArray();
            pattern.listProperties(DCTerms.type).forEachRemaining(stmt -> array.add(stmt.getResource().getURI()));
            obj.put("categories", array);
        }
        if (pattern.hasProperty(DP.icon)) {
            obj.put("icon", pattern.getPropertyResourceValue(DP.icon).getLocalName());
        }
        if (pattern.hasProperty(DCTerms.subject)) {
            obj.put("subject", pattern.getProperty(DCTerms.subject).getString());
        }
        if (pattern.hasProperty(DCTerms.description)) {
            obj.put("description", pattern.getProperty(DCTerms.description).getString());
        }
        if (pattern.hasProperty(DP.context)) {
            obj.put("context", pattern.getProperty(DP.context).getString());
        }
        if (pattern.hasProperty(DP.solution)) {
            obj.put("solution", pattern.getProperty(DP.solution).getString());
        }
        if (pattern.hasProperty(DCTerms.license)) {
            obj.put("license", pattern.getPropertyResourceValue(DCTerms.license).getLocalName());
        }
        if (pattern.hasProperty(FOAF.page)) {
            JsonObject page = new JsonObject();
            Resource p = pattern.getPropertyResourceValue(FOAF.page);
            page.put("topic", p.getProperty(FOAF.topic).getString());
            page.put("primaryTopic", p.getPropertyResourceValue(FOAF.primaryTopic).getURI());
            obj.put("page", page);
        }
        if (pattern.hasProperty(DCTerms.relation)) {
            JsonArray array = new JsonArray();
            pattern.listProperties(DCTerms.relation).forEachRemaining(stmt -> array.add(stmt.getResource().getURI()));
            obj.put("relations", array);
        }
        if (pattern.hasProperty(DP.hasImpactOn)) {
            JsonArray array = new JsonArray();
            pattern.listProperties(DP.hasImpactOn).forEachRemaining(stmt -> array.add(stmt.getResource().getURI()));
            obj.put("impacts", array);
        }
        if (pattern.hasProperty(DP.hasPositiveImpactOn)) {
            JsonArray array = new JsonArray();
            pattern.listProperties(DP.hasPositiveImpactOn).forEachRemaining(stmt -> array.add(stmt.getResource().getURI()));
            obj.put("positive-impacts", array);
        }
        if (pattern.hasProperty(DP.hasNegativeImpactOn)) {
            JsonArray array = new JsonArray();
            pattern.listProperties(DP.hasNegativeImpactOn).forEachRemaining(stmt -> array.add(stmt.getResource().getURI()));
            obj.put("negative-impacts", array);
        }
        return obj;
    }

}
